--########################################################################
--
-- memria na forma de um vetor -
--
--########################################################################
library IEEE;
use ieee.std_logic_1164.all;
use IEEE.Std_Logic_unsigned.all;

entity RAM_mem is
      port( clock, ce, rw : in  std_logic;
            address:  in  std_logic_vector(7 downto 0);
            datain:   in  std_logic_vector(7 downto 0);
            dataout:  out std_logic_vector(7 downto 0) );
end RAM_mem;

architecture RAM_mem of RAM_mem is
   type ram is array (0 to 255) of std_logic_vector(7 downto 0);
   signal memoria : ram := (
                              x"40",
                              x"33",
                              x"54",
                              x"08",
                              x"00",
                              x"00",
                              x"00",
                              x"00",
                              x"AB",
                              others => x"00"
                                                   );
begin    

   process(clock)
   begin
       if clock'event and clock='1' then
          if ce='1' and rw='0' then               
                  memoria(CONV_INTEGER(address)) <= datain;       -- writes in memory
          end if;  
          
          dataout <=  memoria(CONV_INTEGER(address)) ;              -- memory reading
          
       end if;
   end process;    
   
end RAM_mem;



--########################################################################
--
--  TEST BENCH DO BLOCO DE DADOS
--
--########################################################################
library IEEE;
use IEEE.Std_Logic_1164.all;
use work.cleo.all;

entity TB_BLOCO_DADOS  is
end TB_BLOCO_DADOS;

architecture a1 of TB_BLOCO_DADOS is

    signal clock, reset, hold: std_logic;
    
    signal uins : microinstrucao; 
    signal n,z,c,v : std_logic;
    signal address,datain, dataout, ir: internal_bus;
              
begin

   DP: entity work.datapath port map (clock=>clock, reset=>reset,
           hold=>hold, address=>address, datain=>datain, dataout=>dataout,
           ir=>ir, uins=>uins, n=>n, z=>z, c=>c, v=>v);  

   ramC: entity work.RAM_mem port map
               ( clock=>clock, ce=>uins.ce,rw=>uins.rw, address=>address, datain=>dataout, dataout=>datain);
                   
   -- gera o clock
   process
            begin
             clock <= '1', '0' after 5 ns;
             wait for 10 ns;
   end process;
   
   reset <= '1', '0' after 2 ns ; 
        
   process
   begin         
            hold <= '0';
            
            --- primeiro fectch
            uins <= ("000", "011", "111",'0','0','0','0');  -- mar <- pc  
            wait for 10 ns;                                  
            uins <= ("110", "011", "001",'1','1','0','0');  -- mdr <- pmem(mar); pc++
            wait for 10 ns; 
            uins <= ("010", "001", "100",'0','0','0','0');  -- ir  <- mdr
            wait for 10 ns; 
            
            -- primeira execucao
            
            uins <= ("000", "011", "111",'0','0','0','0');  -- mar <- pc  
            wait for 10 ns;                                         
            
            uins <= ("110", "011", "001",'1','1','0','0');  -- mdr <- pmem(mar); pc++
            wait for 10 ns; 
         
            uins <= ("100", "001", "100", '0','0', '1','0'); -- ac <- mdr; lnz
            wait for 10 ns; 

            -- segundo fetch
            uins <= ("000", "011", "111",'0','0','0','0');  -- mar <- pc  
            wait for 10 ns;                                           -- 1 perdo
            uins <= ("110", "011", "001",'1','1','0','0');  -- mdr <- pmem(mar); pc++
            wait for 10 ns; 
            uins <= ("010", "001", "100",'0','0','0','0');  -- ir  <- mdr
            wait for 10 ns; 

            -- segunda execucao
                        
            uins <= ("000", "011", "111",'0','0','0','0');  -- mar <- pc  
            wait for 10 ns;                                 
            
            uins <= ("110", "011", "001",'1','1','0','0');  -- mdr <- pmem(mar); pc++
            wait for 10 ns; 

            uins <= ("000", "001", "100",'0','0','0','0');  -- mar <- mdr 
            wait for 10 ns;                                 
            
            uins <= ("001", "000", "111",'1','1','0','0');  -- mdr <-- pmem (mar)
            wait for 10 ns; 

            uins <= ("100", "110", "000", '0','0', '1','1'); -- ac <- mdr+ac; lnz ; lcv
            wait for 10 ns; 
        
            uins <= ("000", "000", "000", '0','0', '0','0'); 
            hold <= '1';
                        
            wait for 10000 ns;       ---- PAROU
   end process;
   
end a1;

 